/*
    See the JupyterLab Developer Guide for useful CSS Patterns:

    https://jupyterlab.readthedocs.io/en/stable/developer/css.html
*/

body[data-jp-theme-light='true'] [data-id='flare-panel'] svg {
    filter: invert(0);
}
  		  
body[data-jp-theme-light='false'] [data-id='flare-panel'] svg {
    filter: invert(1);
}
